import bcrypt from "bcrypt";
import { Request, Response } from "express";
import Worker from "../db/models/Worker";

//create new account
const createAccountController = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }

    const { first_name, last_name, email, password, role } = req.body;

    const account = await Worker.findAll({
      where: {
        email: email,
      },
    });

    if (account.length > 0) {
      return res
        .status(409)
        .json({ success: false, error: "AccountAlreadyExists" });
    }

    const hashedPassword = await bcrypt.hash(password, 10);

    const newAccount = await Worker.create({
      email,
      firstName: first_name,
      lastName: last_name,
      password: hashedPassword,
      role,
    });

    return res.status(201).send({
      success: true,
      account: {
        first_name: newAccount.firstName,
        last_name: newAccount.lastName,
        email: newAccount.email,
        role: newAccount.role,
      },
    });
  } catch {
    return res.status(500).json({ success: false, error: "serverError" });
  }
};
//show all accounts
const showAllAccounts = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const allAccounts = await Worker.findAll();
    return res.status(200).send({ success: true, result: allAccounts });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//show a specific account using given id: email is the primary key for a worker  
const showAccountById = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const givenId = req.params.id;
    const account = await Worker.findByPk(givenId);
    if (account) {
      return res.status(200).json({ success: true, result: account });
    }
    return res.status(404).json({ success: false, error: "accountIdNotFound" });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};
const accountsControllers = {
  createAccountController,
  showAllAccounts,
  showAccountById,
};

export default accountsControllers;
